"use strict";
// Package conust transforms numbers into string tokens for which the simple string comparison
// produces the same result as the numeric comparison of the original numbers would.
// The input is not limited to decimal numbers, other bases up to base 36 are accepted with the
// restriction that it must be lowercased.
// The expected input format is ^[+-]?[0-9a-z]+(\.[0-9a-z]+)?$ Failing to satisfy this results in encoding failures.
//
// The conversion adds a few characters to the length of the original numeric string, but at
// the same time it can save some space by storing only the significant portion of the number
// omitting trailing and leading zeros of it in the output.
//
// Beside transforming single numbers to sortable strings, you can also transform a string containing both
// text and numbers into a properly sortable version. However, the reverse transformation of such mixed strings
// is not possible.
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.conust = exports.encodeNumber = void 0;
var big_js_1 = __importDefault(require("big.js"));
var maxDigitValue = 35;
var maxMagnitudeDigitValue = 34;
var digit0 = '0'.charCodeAt(0);
var digitA = 'a'.charCodeAt(0);
var digitZ = 'z'.charCodeAt(0);
var digit9 = '9'.charCodeAt(0);
var signNegativeMagPositive = '3';
var signNegativeMagNegative = '4';
var zeroOutput = '5';
var signPositiveMagNegative = '6';
var signPositiveMagPositive = '7';
var negativeNumberTerminator = '~';
var digits36 = [
    '0',
    '1',
    '2',
    '3',
    '4',
    '5',
    '6',
    '7',
    '8',
    '9',
    'a',
    'b',
    'c',
    'd',
    'e',
    'f',
    'g',
    'h',
    'i',
    'j',
    'k',
    'l',
    'm',
    'n',
    'o',
    'p',
    'q',
    'r',
    's',
    't',
    'u',
    'v',
    'w',
    'x',
    'y',
    'z',
];
var digits36Reversed = [
    'z',
    'y',
    'x',
    'w',
    'v',
    'u',
    't',
    's',
    'r',
    'q',
    'p',
    'o',
    'n',
    'm',
    'l',
    'k',
    'j',
    'i',
    'h',
    'g',
    'f',
    'e',
    'd',
    'c',
    'b',
    'a',
    '9',
    '8',
    '7',
    '6',
    '5',
    '4',
    '3',
    '2',
    '1',
    '0',
];
function encodeNumber(input) {
    if ('string' !== typeof input && 'number' !== typeof input) {
        throw new Error("expected typeof input to be string, but received \"".concat(typeof input, "\""));
    }
    var numberString = typeof input === 'number' ? (0, big_js_1.default)(input).toFixed() : input;
    try {
        if (!numberString.length)
            return '';
        var positive = numberString[0] !== '-';
        var startPos = getSignificantStartPos(numberString);
        var endPos = getSignificantEndPos(numberString);
        if (startPos === endPos)
            return zeroOutput;
        var decimalPointPos = numberString.indexOf('.');
        var _a = getMagnitudeParams(numberString.length, startPos, endPos, decimalPointPos), magnitudePositive = _a.magnitudePositive, magnitude = _a.magnitude;
        var signDigit = encodeSign(positive, magnitudePositive);
        var digit2 = writeMagnitude(positive, magnitudePositive, magnitude);
        var result = "".concat(signDigit).concat(digit2);
        if (startPos < decimalPointPos && decimalPointPos < endPos) {
            result += writeDigits(positive, numberString.substring(startPos, decimalPointPos));
            result += writeDigits(positive, numberString.substring(decimalPointPos + 1, endPos));
        }
        else {
            result += writeDigits(positive, numberString.substring(startPos, endPos));
        }
        if (!positive) {
            result += negativeNumberTerminator;
        }
        return result;
    }
    catch (e) {
        e.stack = "parse(".concat(typeof numberString, " \"").concat(numberString, "\"): ").concat(e.stack);
        throw e;
    }
}
exports.encodeNumber = encodeNumber;
function conust(input) {
    return encodeNumber(input);
}
exports.conust = conust;
function writeDigits(positive, digits) {
    if (positive) {
        return digits;
    }
    else {
        var result = '';
        for (var i = 0; i < digits.length; i++) {
            var int = digitToInt(digits.charCodeAt(i));
            var digit = digits36Reversed[int];
            result += digit;
        }
        return result;
    }
}
function digitToInt(digit) {
    if (digit < digitA) {
        return digit - digit0;
    }
    return 10 + (digit - digitA);
}
function encodeSign(positive, magnitudePositive) {
    if (positive) {
        if (magnitudePositive) {
            return signPositiveMagPositive;
        }
        return signPositiveMagNegative;
    }
    if (magnitudePositive) {
        return signNegativeMagPositive;
    }
    return signNegativeMagNegative;
}
function getMagnitudeParams(inputLength, sStartPos, _sEndPos, decimalPointPos) {
    var magnitude;
    var magnitudePositive;
    if (decimalPointPos < 0) {
        magnitude = inputLength - sStartPos;
        magnitudePositive = true;
    }
    else if (decimalPointPos < sStartPos) {
        magnitude = sStartPos - (decimalPointPos + 1);
        magnitudePositive = false;
    }
    else {
        magnitude = decimalPointPos - sStartPos;
        magnitudePositive = true;
    }
    return { magnitude: magnitude, magnitudePositive: magnitudePositive };
}
function writeMagnitude(positive, magnitudePositive, magnitude) {
    var reverseDigits = positive != magnitudePositive;
    var result = '';
    while (magnitude > maxMagnitudeDigitValue) {
        magnitude -= maxMagnitudeDigitValue;
        if (reverseDigits) {
            result += digits36Reversed[maxDigitValue];
        }
        else {
            result += digits36[maxDigitValue];
        }
    }
    if (reverseDigits) {
        result += digits36Reversed[magnitude];
    }
    else {
        result += digits36[magnitude];
    }
    return result;
}
function getSignificantStartPos(input) {
    var i = 0;
    while (i < input.length) {
        if (isDigit(input[i]) && input[i] !== '0') {
            return i;
        }
        ++i;
    }
    return -1;
}
function getSignificantEndPos(input) {
    var i = input.length - 1;
    while (i >= 0) {
        if (isDigit(input[i]) && input[i] !== '0') {
            return i + 1;
        }
        --i;
    }
    return -1;
}
function isDigit(char) {
    var digit = char.charCodeAt(0);
    return ((digit >= digit0 && digit <= digit9) || (digit >= digitA && digit <= digitZ));
}
